/*
 * Copyright (C) 2014 digitalSTROM.org, Schlieren, Switzerland
 *
 * Authors: Andreas Fenkart, <andreas.fenkart@dev.digitalstrom.org>
 *
 * digitalSTROM ds485-client library is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the License,
 * or (at your option) any later version.
 *
 * digitalSTROM ds485-client library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with digitalSTROM ds485-client library. If not, see
 * <http://www.gnu.org/licenses/>.
 */

#include "socket_send.h"

#include <errno.h>
#include <sys/socket.h>
#include <time.h>

#if defined(__APPLE__) || defined(__MACH__)
# ifndef MSG_NOSIGNAL
#   define MSG_NOSIGNAL SO_NOSIGPIPE
# endif
#endif

int socket_send(int socket, const uint8_t* data, size_t dlen)
{
    struct timespec tim = { 0, 50000000 }; /* 50 ms */
    ssize_t nbytes, todo = dlen;
    int retries = 0;

    while (retries < 3 && todo > 0) {
        nbytes = send(socket, data, todo, MSG_NOSIGNAL);
        if (nbytes == -1) {
            if (errno == EAGAIN || errno == EWOULDBLOCK || errno == EINTR) {
                retries++;
                nanosleep(&tim , NULL);
                continue;
            }
            return errno; /* peer has disconnected */
        }
        data += nbytes;
        todo -= nbytes;
    }

    if (todo > 0) {
        return errno;
    }
    return 0;
}

int socket_writev(int socket, const struct iovec *iov, int iovcnt)
{
    int i, err;

    for (i = 0; i < iovcnt; i++) {
        /* TODO writev is hard to use with retries */
        err = socket_send(socket, iov[i].iov_base, iov[i].iov_len);
        if (err) {
            return err;
        }
    }
    return 0;
}
